/*****************************************************************************\
* Copyright (c) 2002 Pelle Johansson.                                         *
* All rights reserved.                                                        *
*                                                                             *
* This file is part of the moftpd package. Use and distribution of            *
* this software is governed by the terms in the file LICENCE, which           *
* should have come with this package.                                         *
* Notice that after passing this file through a yacc parser, you may          *
* also have to consider the licence of its output.                            *
\*****************************************************************************/

/* $moftpd: config.y,v 1.21.2.10 2004/03/18 15:10:02 morth Exp $ */
/* $Id$ */

%{

#include "confparse.h"

#include "server.h"
#include "user.h"
#include "file.h"
#include "memory.h"

static struct stack
{
  void *item;
  struct stack *next;
} *servstack, *userstack, *dirstack, *servtop, *usertop, *dirtop;

static server_t *currserv;
static user_t *curruser;
static access_t *currdir;

int confExpectingToken;

extern int unprivUid, unprivGid;
extern int xferBufferSize, fakeChroot, forkConnections, maxMmapSize;
extern const char *localeDir;
#ifdef HAVE_LIBSSL
extern char *sslCertsPath;
#endif

%}

%union
{
  long long num;
  char str[100];
}

%token <num> D_NUMBER
%token <str> D_STRING
%token <num> D_BOOL
%token <num> D_UID
%token <num> D_GID

%token I_SERVER I_USER I_DIRECTORY
%token I_SQLSERVER I_SQLUSER I_SQLDIRECTORY

%token I_SQLCONNECT I_UNPRIVUID I_UNPRIVGID I_XFERBUFFSIZE I_FAKECHROOT
%token I_FORKCLIENTS I_RESET I_EXEC I_LOCALEDIR I_TRUSTEDCERTSDIR I_MAXMMAPSIZE

%token I_PORT I_BIND I_MINPASVPORT I_MAXPASVPORT I_PASSIFINVALID I_ALLOWUNBOUND
%token I_ALLOWFOREIGN I_ALLOWLOWPORTS I_USERALIAS I_SLEEPONFAIL
%token I_MAXLOGINATTEMPTS
%token I_ACCEPTTLS I_TLSVERIFYCLIENT I_TLSAUTOLOGIN I_TLSNONEWUSER

%token I_LOGINFAILEDMSG I_USERINVALIDMSG I_PASSREQUESTMSG I_ANONPASSMSG

%token I_ANONYMOUS I_HOME I_CHROOT I_ALIAS I_PASSWORDNEEDED I_PASSWORD I_UID
%token I_GID I_ALLOWLOGIN I_MAXIDLE I_ALLOWSECLOGIN

%token I_ALLOW I_DENY I_HIDDEN I_FAKEUSER I_FAKEGROUP I_REQUIRE

%token I_SEARCH I_READFILE I_LISTING I_CREATEFILE I_CREATEDIR I_APPEND I_OVERWRITE
%token I_FAKEFILEMODE I_FAKEDIRMODE
%token I_DELETE I_RENAME I_READING I_WRITING I_STORING I_ALL I_ENCRYPTED I_SIGNED

%token I_ETAG

%type <num> bool boolExec accesslist accessitem

%%

input:		/* empty */
		| input topst
		;

space:		/*
		 * empty - space is obsolete, but has to be
		 * removed from all rules below.
		 */
		;

semicolon:	/* empty */
		| ';'
		;

bool:		D_BOOL
		{ $$ = $1; }
		| D_NUMBER
		{ $$ = $1; }
		;

topst:		topopt semicolon
		| server
		;

topopt:		I_SQLCONNECT space D_NUMBER space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
#if 0
		  if(sql_connect($3, $8, $13, $18, $23))
		    YYABORT;
#endif
		}
		| I_UNPRIVUID space
		{ confExpectingToken = D_UID; }
		D_UID
		{
		  unprivUid = $4;
		}
		| I_UNPRIVGID space
		{ confExpectingToken = D_GID; }
		D_GID
		{
		  unprivGid = $4;
		}
		| I_XFERBUFFSIZE space D_NUMBER
		{
		  if($3 <= 0)
		  {
		    syslog (LOG_ERR, "XferBuffSize must be positive.");
		    YYABORT;
		  }
		  xferBufferSize = $3;
		}
		| I_FAKECHROOT space bool
		{
		  fakeChroot = $3;
		}
		| I_FORKCLIENTS space boolExec
		{
		  forkConnections = $3;
		}
		| I_LOCALEDIR space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if ($4[strlen ($4) - 1] != '/')
		  {
		    char *tmp = talloc (strlen ($4) + 2);
		    if (tmp)
		    {
		      strcpy (tmp, $4);
		      strcat (tmp, "/");
		      localeDir = pstring (tmp, NULL);
		    }
		  }
		  else
		    localeDir = pstring ($4, NULL);
		}
		| I_TRUSTEDCERTSDIR space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
#ifdef HAVE_LIBSSL
		  pfree (sslCertsPath, NULL);
		  sslCertsPath = pstring ($4, NULL);
#endif
		}
		| I_MAXMMAPSIZE D_NUMBER
		{
		  maxMmapSize = $2;
		}
		;

boolExec:	bool
		{
		  if($1)
		    $$ = 1;
		  else
		    $$ = 0;
		}
		| I_EXEC
		{
		  $$ = 2;
		}
		;

server:		servstart servdata servend
		| I_SQLSERVER space D_NUMBER space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING semicolon
		{
#if 0
		  const char *str = sql_parse_servers($3, $8);
		  
		  if(str)
		    insert_config_data(str);
		  else
		    YYABORT;
#endif
		}
		;

servstart:	'<' space I_SERVER space
		{ confExpectingToken = D_STRING; }
		D_STRING space '>'
		{
		  currserv = new_server($6, currserv);
		  servtop = talloc (sizeof (*servtop));
		  
		  if(!currserv || !servtop)
		    YYABORT;
		  
		  servtop->item = currserv;
		  servtop->next = servstack;
		  servstack = servtop;
		}
		;

servend:	I_ETAG space I_SERVER space '>'
		{
		  servstack = servstack->next;
		  currserv = servstack? servstack->item : NULL;
		}
		;

servdata:	/* empty */
		| servdata servst
		;

servst:		server
		| servopt semicolon
		| user
		| directory
		;

servopt:	I_PORT space D_NUMBER
		{
		  add_server_port(currserv, $3);
		}
		| I_BIND space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  add_server_binding(currserv, $4);
		}
		| I_MINPASVPORT space D_NUMBER
		{
		  if($3 < 0)
		    $3 = 0;
		  if($3 > 65535)
		    $3 = 65535;
		  currserv->minPasvPort = $3;
		  if(currserv->maxPasvPort < currserv->minPasvPort)
		    currserv->maxPasvPort = currserv->minPasvPort;
		}
		| I_MAXPASVPORT space D_NUMBER
		{
		  if($3 < 0)
		    $3 = 0;
		  if($3 > 65535)
		    $3 = 65535;
		  currserv->maxPasvPort = $3;
		  if(currserv->minPasvPort > currserv->maxPasvPort)
		    currserv->minPasvPort = currserv->maxPasvPort;
		}
		| I_PASSIFINVALID space bool
		{
		  currserv->passIfInvalid = $3;
		}
		| I_ALLOWUNBOUND space bool
		{
		  currserv->allowUnbound = $3;
		}
		| I_ALLOWFOREIGN space bool
		{
		  currserv->allowForeign = $3;
		}
		| I_ALLOWLOWPORTS space bool
		{
		  currserv->allowLowPorts = $3;
		}
		| I_CHROOT space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(currserv->chroot)
		    pfree(currserv->chroot, currserv);
		  currserv->chroot = pstring($4, currserv);
		}
		| I_PASSWORDNEEDED space bool
		{
		  currserv->passwordNeeded = ($3 != 0);
		}
		| I_ALLOWLOGIN space bool
		{
		  currserv->allowLogin = ($3 != 0);
		}
		| I_ALLOWSECLOGIN space bool
		{
		  currserv->allowSecLogin = ($3 != 0);
		}
		| I_LOGINFAILEDMSG space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(currserv->loginFailedMsg)
		    pfree(currserv->loginFailedMsg, currserv);
		  currserv->loginFailedMsg = pstring($4, currserv);
		}
		| I_USERINVALIDMSG space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(currserv->userInvalidMsg)
		    pfree(currserv->userInvalidMsg, currserv);
		  currserv->userInvalidMsg = pstring($4, currserv);
		}
		| I_PASSREQUESTMSG space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(currserv->passRequestMsg)
		    pfree(currserv->passRequestMsg, currserv);
		  currserv->passRequestMsg = pstring($4, currserv);
		}
		| I_ANONPASSMSG space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(currserv->anonPassMsg)
		    pfree(currserv->anonPassMsg, currserv);
		  currserv->anonPassMsg = pstring($4, currserv);
		}
		| I_MAXIDLE space D_NUMBER
		{
		  currserv->maxIdle = $3;
		}
		| I_ACCEPTTLS space
		{ confExpectingToken = D_STRING; }
		D_STRING space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
#ifdef HAVE_LIBSSL
		  FILE *fp;
		  if ($4[0] != '/')
		  {
		    char *tmp = talloc (strlen (sslCertsPath) + strlen ($4) + 2);
		    
		    if (tmp)
		    {
		      strcpy (tmp, sslCertsPath);
		      strcat (tmp, "/");
		      strcat (tmp, $4);
		      fp = fopen (tmp, "rb");
		    }
		    else
		      fp = NULL;
		  }
		  else
		    fp = fopen ($4, "rb");
		  if (!fp)
		  {
		    syslog (LOG_ERR, "Failed to open certificate file: %m");
		    YYABORT;
		  }
		  currserv->tlsCert = PEM_read_X509 (fp, NULL, NULL, NULL);
		  fclose (fp);
		  if (!currserv->tlsCert)
		  {
		    syslog (LOG_ERR, "Error reading certificate.");
		    YYABORT;
		  }
		  
		  if ($9[0] != '/')
		  {
		    char *tmp = talloc (strlen (sslCertsPath) + strlen ($9) + 2);
		    
		    if (tmp)
		    {
		      strcpy (tmp, sslCertsPath);
		      strcat (tmp, "/");
		      strcat (tmp, $9);
		      fp = fopen (tmp, "rb");
		    }
		    else
		      fp = NULL;
		  }
		  else
		    fp = fopen ($9, "rb");
		  if (!fp)
		  {
		    syslog (LOG_ERR, "Failed to open key file: %m");
		    YYABORT;
		  }
		  currserv->tlsKey = PEM_read_PrivateKey (fp, NULL, NULL, NULL);
		  fclose (fp);
		  if (!currserv->tlsKey)
		  {
		    syslog (LOG_ERR, "Error reading kye.");
		    YYABORT;
		  }
#else
		  syslog (LOG_WARNING, "Warning: AcceptTLS used but TLS "
			"support is not compiled.");
#endif
		}
		| I_TLSVERIFYCLIENT space D_BOOL
		{
		  currserv->verifyTLS = ($3 != 0);
		}
		| I_TLSAUTOLOGIN space D_BOOL
		{
		  currserv->loginTLS = ($3 != 0);
		}
		| I_TLSNONEWUSER space D_BOOL
		{
		  currserv->tlsNoLogout = ($3 != 0);
		}
		| I_USERALIAS space
		{ confExpectingToken = D_STRING; }
		D_STRING space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  server_add_alias (currserv, $9, $4);
		}
		| I_SLEEPONFAIL D_NUMBER
		{
		  currserv->sleepOnFail = $2;
		}
		| I_MAXLOGINATTEMPTS D_NUMBER
		{
		  currserv->maxLoginAttempts = $2;
		}
		| I_RESET space srvrstlist
		;

srvrstlist:	srvrstitem
		| srvrstlist ',' space srvrstitem
		;

srvrstitem:	I_PORT
		{
		  if(currserv->ports)
		    pfree(currserv->ports, currserv);
		  currserv->ports = NULL;
		  currserv->numPorts = 0;
		}
		| I_BIND
		{
		  if(currserv->bindings)
		    pfree(currserv->bindings, currserv);
		  currserv->bindings = NULL;
		  currserv->numBindings = 0;
		}
		| I_LOGINFAILEDMSG
		{
		  if(currserv->loginFailedMsg)
		    pfree(currserv->loginFailedMsg, currserv);
		  currserv->loginFailedMsg = NULL;
		}
		| I_USERINVALIDMSG
		{
		  if(currserv->userInvalidMsg)
		    pfree(currserv->userInvalidMsg, currserv);
		  currserv->userInvalidMsg = NULL;
		}
		| I_PASSREQUESTMSG
		{
		  if(currserv->passRequestMsg)
		    pfree(currserv->passRequestMsg, currserv);
		  currserv->passRequestMsg = NULL;
		}
		| I_ANONPASSMSG
		{
		  if(currserv->anonPassMsg)
		    pfree(currserv->anonPassMsg, currserv);
		  currserv->anonPassMsg = NULL;
		}
		;

user:		userstart userdata userend
		| I_SQLUSER space D_NUMBER space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING semicolon
		{
#if 0
		  const char *str = sql_parse_users($3, $8);
		  
		  if(str)
		    insert_config_data(str);
		  else
		    YYABORT;
#endif
		}
		;

userstart:	'<' space I_USER space
		{ confExpectingToken = D_STRING; }
		D_STRING space '>'
		{
		  if (curruser)
		  {
		    curruser = copy_user (curruser, $6, currserv);
		    if (!curruser)
		      YYABORT;
		  }
		  else
		  {
		    curruser = find_server_user (currserv, $6, currserv);
		    if (!curruser)
		    {
		      curruser = new_user ($6, currserv);
		      curruser->allowLogin = currserv->allowLogin;
		      curruser->allowSecLogin = currserv->allowSecLogin;
		    }
		    if (!curruser)
		      YYABORT;
		    pfree (curruser->access, curruser);
		    curruser->access = pattach(currserv->access, curruser);
		    curruser->maxIdle = currserv->maxIdle;
		    curruser->passwordNeeded = currserv->passwordNeeded;
		    // Reset uid and gid, but don't activate in spec_flags.
		    curruser->uid = unprivUid;
		    curruser->gid = unprivGid;
		    if(currserv->chroot)
		    {
		      pfree (curruser->chroot, curruser);
		      curruser->chroot = pattach(currserv->chroot, curruser);
		      curruser->spec_flags |= ufChroot;
		    }
		  }
		  usertop = talloc (sizeof (*usertop));
		  if (!usertop)
		    YYABORT;
		  usertop->item = curruser;
		  usertop->next = userstack;
		  userstack = usertop;
		}
		;

userend:	I_ETAG space I_USER space '>'
		{
		  if(curruser->chroot)
		  {
		    char *tmp = curruser->chroot;
		    char *home = curruser->home;
		    user_t *pwUser = NULL;
		    
		    if (!home)
		    {
		      pwUser = find_passwd_user (curruser->name, NULL);
		      if (pwUser && pwUser->home)
			home = pwUser->home;
		    }
		    curruser->chroot = pstring(full_path(curruser->chroot,
			  home), curruser);
		    pfree(tmp, curruser);
		    pfree (pwUser, NULL);
		  }
		  curruser->next = padopt (currserv->users, currserv, curruser);
		  currserv->users = curruser;
		  userstack = userstack->next;
		  curruser = userstack? userstack->item : NULL;
		}
		;

userdata:	/* empty */
		| userdata userst
		;

userst:		user
		| useropt semicolon
		| directory
		;

useropt:	I_ANONYMOUS space bool
		{
		  curruser->anonymous = ($3 != 0);
		  curruser->spec_flags |= ufAnonymous;
		  if(curruser->anonymous)
		  {
		    curruser->allowLogin = 1;
		    curruser->passwordNeeded = 1;
		  }
		}
		| I_ALIAS space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  server_add_alias(currserv, $4, curruser->name);
		}
		| I_HOME space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(curruser->home)
		    pfree(curruser->home, curruser);
		  curruser->home = pstring($4, curruser);
		  curruser->spec_flags |= ufHome;
		}
		| I_CHROOT space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(curruser->chroot)
		    pfree(curruser->chroot, curruser);
		  curruser->chroot = pstring($4, curruser);
		  curruser->spec_flags |= ufChroot;
		}
		| I_PASSWORDNEEDED space bool
		{
		  curruser->passwordNeeded = ($3 != 0);
		}
		| I_PASSWORD space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  if(curruser->password)
		    pfree(curruser->password, curruser);
		  curruser->spec_flags |= ufPassword | ufAnonymous;
		  curruser->password = pstring($4, curruser);
		  curruser->passwordNeeded = 1;
		  curruser->anonymous = 0;
		}
		| I_UID space
		{ confExpectingToken = D_UID; }
		D_UID
		{
		  curruser->spec_flags |= ufUid;
		  curruser->uid = $4;
		}
		| I_GID space
		{ confExpectingToken = D_GID; }
		D_GID
		{
		  curruser->spec_flags |= ufGid;
		  curruser->gid = $4;
		}
		| I_ALLOWLOGIN space bool
		{
		  curruser->allowLogin = ($3 != 0);
		}
		| I_ALLOWSECLOGIN space bool
		{
		  curruser->allowSecLogin = ($3 != 0);
		}
		| I_MAXIDLE space D_NUMBER
		{
		  curruser->maxIdle = $3;
		}
		| I_RESET space usrrstlist
		;

usrrstlist:	usrrstitem
		| usrrstlist ',' space usrrstitem
		;

usrrstitem:	I_UID
		{
		  curruser->spec_flags &= ~ufUid;
		}
		| I_GID
		{
		  curruser->spec_flags &= ~ufGid;
		}
		| I_HOME
		{
		  curruser->spec_flags &= ~ufHome;
		  if(curruser->home)
		    pfree(curruser->home, curruser);
		  curruser->home = NULL;
		}
		| I_PASSWORD
		{
		  curruser->spec_flags &= ~ufPassword;
		}
		;

directory:	dirstart dirdata dirend
		| I_SQLDIRECTORY space D_NUMBER space ',' space
		{ confExpectingToken = D_STRING; }
		D_STRING semicolon
		{
#if 0
		  const char *str = sql_parse_directories($3, $8);
		  
		  if(str)
		    insert_config_data(str);
		  else
		    YYABORT;
#endif
		}
		;

dirstart:	'<' space I_DIRECTORY space
		{ confExpectingToken = D_STRING; }
		D_STRING space '>'
		{
		  access_t *p;
		  
		  p = currdir;
		  currdir = palloc(sizeof(access_t), NULL);
		  if(!currdir)
		    YYABORT;
		  if($6[0] == '/' || $6[0] == '~')
		    currdir->path = pstring(full_path($6, curruser?
			  curruser->home : NULL), currdir);
		  else
		  {
		    char *buf;
		    
		    if(p)
		      buf = talloc (strlen (p->path) + strlen ($6) + 2);
		    else
		      buf = talloc (strlen ($6) + 2);
		    if (buf)
		    {
		      if (p)
			strcpy (buf, p->path);
		      if(buf[strlen(buf) - 1] != '/')
			strcat(buf, "/");
		      strcat(buf, $6);
		      currdir->path = pstring (full_path(buf, NULL), currdir);
		    }
		  }
		  currdir->deny = currdir->allow = 0;
		  currdir->fakeUser = NULL;
		  currdir->fakeGroup = NULL;
		  if(curruser)
		  {
		    currdir->next = padopt(curruser->access, curruser, currdir);
		    curruser->access = pattach(currdir, curruser);
		  }
		  else
		  {
		    currdir->next = padopt(currserv->access, currserv, currdir);
		    currserv->access = pattach(currdir, currserv);
		  }
		  dirtop = talloc (sizeof (*dirtop));
		  if (!dirtop)
		    YYABORT;
		  dirtop->item = currdir;
		  dirtop->next = dirstack;
		  dirstack = dirtop;
		}
		;

dirend:		I_ETAG space I_DIRECTORY space '>'
		{
		  dirstack = dirstack->next;
		  currdir = dirstack? dirstack->item : NULL;
		}
		;

dirdata:	/* empty */
		| dirdata dirst
		;

dirst:		directory
		| diropt semicolon
		;

diropt:		I_ALLOW accesslist
		{
		  currdir->deny &= ~$2;
		  currdir->require &= ~$2;
		  currdir->allow |= $2;
		}
		| I_DENY accesslist
		{
		  currdir->allow &= ~$2;
		  currdir->require &= ~$2;
		  currdir->deny |= $2;
		}
		| I_REQUIRE accesslist
		{
		  $2 &= acEncrypted | acSigned;
		  
		  currdir->allow &= ~$2;
		  currdir->deny &= ~$2;
		  currdir->require |= $2;
		}
		| I_HIDDEN space bool
		{
		  currdir->hidden = ($3 != 0);
		}
		| I_FAKEUSER space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  pfree(currdir->fakeUser, currdir);
		  currdir->fakeUser = pstring($4, currdir);
		}
		| I_FAKEGROUP space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  pfree(currdir->fakeGroup, currdir);
		  currdir->fakeGroup = pstring($4, currdir);
		}
		| I_FAKEFILEMODE space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  pfree(currdir->fakeFile, currdir);
		  currdir->fakeFile = pstring($4, currdir);
		}
		| I_FAKEDIRMODE space
		{ confExpectingToken = D_STRING; }
		D_STRING
		{
		  pfree(currdir->fakeDir, currdir);
		  currdir->fakeDir = pstring($4, currdir);
		}
		| I_RESET space dirrstlist
		;

dirrstlist:	dirrstitem
		| dirrstlist ',' space dirrstitem
		;

dirrstitem:	I_FAKEUSER
		{
		  if(currdir->fakeUser != (char*)-1)
		    pfree(currdir->fakeUser, currdir);
		  currdir->fakeUser = (char*)-1;
		}
		| I_FAKEGROUP
		{
		  if(currdir->fakeGroup != (char*)-1)
		    pfree(currdir->fakeGroup, currdir);
		  currdir->fakeGroup = (char*)-1;
		}
		| I_FAKEFILEMODE
		{
		  if(currdir->fakeFile != (char*)-1)
		    pfree(currdir->fakeFile, currdir);
		  currdir->fakeFile = (char*)-1;
		}
		| I_FAKEDIRMODE
		{
		  if(currdir->fakeDir != (char*)-1)
		    pfree(currdir->fakeDir, currdir);
		  currdir->fakeDir = (char*)-1;
		}
		;

accesslist:	accessitem
		{
		  $$ = $1;
		}
		| accesslist ',' space accessitem
		{
		  $$ = $1 | $4;
		}
		;

accessitem:	I_SEARCH
		{
		  $$ = acSearch;
		}
		| I_READFILE
		{
		  $$ = acCreateFile;
		}
		| I_LISTING
		{
		  $$ = acListing;
		}
		| I_CREATEFILE
		{
		  $$ = acCreateFile;
		}
		| I_CREATEDIR
		{
		  $$ = acCreateFile;
		}
		| I_APPEND
		{
		  $$ = acAppend;
		}
		| I_OVERWRITE
		{
		  $$ = acOverwrite;
		}
		| I_DELETE
		{
		  $$ = acDelete;
		}
		| I_RENAME
		{
		  $$ = acRename;
		}
		| I_ENCRYPTED
		{
		  $$ = acEncrypted;
		}
		| I_SIGNED
		{
		  $$ = acSigned;
		}
		| I_READING
		{
		  $$ = acSearch | acReadFile | acListing;
		}
		| I_WRITING
		{
		  $$ = acCreateFile | acCreateDir | acAppend | acOverwrite |
		        acDelete;
		}
		| I_STORING
		{
		  $$ = acCreateFile | acCreateDir | acAppend;
		}
		| I_ALL
		{
		  $$ = -1;
		}
		;

%%
