/*****************************************************************************\
* Copyright (c) 2002 Pelle Johansson.                                         *
* All rights reserved.                                                        *
*                                                                             *
* This file is part of the moftpd package. Use and distribution of            *
* this software is governed by the terms in the file LICENCE, which           *
* should have come with this package.                                         *
\*****************************************************************************/

/* $moftpd: connection.h,v 1.18 2003/09/14 13:32:23 morth Exp $ */
/* $Id$ */

#ifndef _CONNECTION_H
#define _CONNECTION_H

typedef struct connection connection_t;

#include "server.h"
#include "user.h"

#include "system.h"

enum
{
  ttText = 0,
  ttImage = 1
};

enum
{
  NUL, SE = 240, SB = 250, WILL, WONT, DO, DONT, IAC
};

enum
{
  tfCR     = 0x1,
  tfSubOpt = 0x2
};

struct connection
{
  int sock;
  char readBuffer[4096];
  int readBufferLen;
  int overFlow:1, authed:1, passive:1, epsvOnly:1, working:1, spontQuit:1,
	passiveAccepted:1, sending:1;
  user_t *user;
  char *email;
  const char *expect;
  char *cwd;
  struct sockaddr_storage lDataAddr, rDataAddr;
  int activePort;
  int dataSock, fileFd;
  char *spontReply;
  int type, subtype;
  unsigned long long restart;
  void *data;
  unsigned long long dataOffset, dataLen;
  char *filePath;
  time_t transferStart;
  char *rnfr;
  time_t lastCommand;
  int telnetMode, telnetFlags;
  unsigned char sb;
#if 0 // Will probably never parse subopts. RFC 1123 says not to.
  char subopt[100];
  int suboptLen;
#endif
  char *currLang;
  int langFd;
#ifdef HAVE_LIBSSL
  SSL *ssl, *sslData;
  int sslState, sslDataState;
#endif
  int pbsz, failedLogins;
  time_t sleepUntil;
  int protLevel;
  int mlstTags;
  
  server_t *server;
};

connection_t *new_connection(int sock, server_t *serv);
void disconnected(connection_t *conn);
void quit_all_connections(const char *str);
void disconnect_all_connections(void);
void check_idle(void);

void reply(connection_t *conn, const char *format, ...)
     __attribute__((format (printf, 2, 3)));
void reply_spont(connection_t *conn, const char *str, int closeConn);
void send_telnet(connection_t *conn, const char *str, int newline);

void control_handler(int sock, void *user, int urgent);
void run_commands (connection_t *conn);
int parse_telnet(connection_t *conn, char *buff, int len);

void authenticated(connection_t *conn);
void unauthenticated(connection_t *conn);

void passive_acceptor(int sock, void *user, int urgent);
int open_data_connection(connection_t *conn);
int close_data_connection(connection_t *conn);
void data_receiver(int sock, void *user, int urgent);
void data_sender(int sock, void *user, int urgent);

#endif /*_CONNECTION_H*/
