/*****************************************************************************\
* Copyright (c) 2002 Pelle Johansson.                                         *
* All rights reserved.                                                        *
*                                                                             *
* This file is part of the moftpd package. Use and distribution of            *
* this software is governed by the terms in the file LICENCE, which           *
* should have come with this package.                                         *
\*****************************************************************************/

/* $moftpd: server.h,v 1.11.2.1 2003/09/18 20:14:20 morth Exp $ */
/* $Id$ */

#ifndef _SERVER_H
#define _SERVER_H

typedef struct server server_t;

#include "user.h"
#include "utf8fs/file.h"

struct sockaddr;

struct server
{
  char *name;
  int numBindings;
  char **bindings;
  int numPorts;
  int *ports;
  int minPasvPort, maxPasvPort;
  user_t *users;
  access_t *access;
  char *chroot;
  char *loginFailedMsg, *userInvalidMsg;
  int maxIdle;
  char *passRequestMsg, *anonPassMsg;
  int tlsOptions;
#ifdef HAVE_LIBSSL
  X509 *tlsCert;
  EVP_PKEY *tlsKey;
#endif
  int sleepOnFail, maxLoginAttempts;
  
  unsigned int passIfInvalid:1, allowUnbound:1, allowForeign:1, allowLowPorts:1,
    allowLogin:1, passwordNeeded:1, verifyTLS:1, loginTLS:1, tlsNoLogout:1,
    allowSecLogin:1;
  
  struct
  {
    char *alias;
    char *user;
  } *aliases;
  int numAliases;
};

server_t *new_server(const char *name, server_t *parent);
void free_server(server_t *serv);
void quit_all_servers();

void add_server_port(server_t *serv, int port);
void add_server_binding(server_t *serv, const char *mask);

void server_add_alias(server_t *serv, const char *alias, const char *user);
const char *server_expand_alias(const server_t *serv, const char *name);

server_t *find_server(const struct sockaddr *addr, int addrlen);
server_t *find_named_server(char *host, int port);
int create_server_sockets(void);
void connection_accepter(int sock, void *user, int urgent);

#endif /*_SERVER_H*/
