/*****************************************************************************\
* Copyright (c) 2002 Pelle Johansson.                                         *
* All rights reserved.                                                        *
*                                                                             *
* This file is part of the moftpd package. Use and distribution of            *
* this software is governed by the terms in the file LICENCE, which           *
* should have come with this package.                                         *
\*****************************************************************************/

/* $moftpd: user.h,v 1.11.2.1 2003/09/16 11:12:28 morth Exp $ */
/* $Id$ */

#ifndef _USER_H
#define _USER_H

typedef struct user user_t;

#include "server.h"
#include "utf8fs/file.h"

#include "system.h"

enum
{
  ufPassword       = 1 << 0,
  ufHome           = 1 << 1,
  ufUid            = 1 << 2,
  ufGid            = 1 << 3,
  ufAnonymous      = 1 << 4,
  ufChroot         = 1 << 5
};

struct user
{
  int spec_flags;
  
  char *name, *password, *home;
  uid_t uid;
  gid_t gid;
  unsigned int allowLogin:1, anonymous:1, passwordNeeded:1, allowSecLogin:1;
  char *chroot;
  access_t *access;
  int maxIdle;
  int ngroups;
  gid_t groups[NGROUPS_MAX];
  
  user_t *next;
};

user_t *new_user(const char *name, void *parent);
user_t *copy_user(const user_t *u, const char *newname, void *newparent);

user_t *find_user(const server_t *serv, const char *name, int allowNoLogin, void *newparent);
user_t *find_passwd_user(const char *name, void *newparent);
user_t *find_server_user(const server_t *server, const char *name, void *newparent);

const char *user_pass(const user_t *user);

void user_setup_environ(const user_t *user, int authed);

#endif /*_USER_H*/
